@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2003 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2004 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -56,6 +56,7 @@ from Mailman.HTMLFormatter import HTMLFormatter
 from Mailman.ListAdmin import ListAdmin
 from Mailman.SecurityManager import SecurityManager
 from Mailman.TopicMgr import TopicMgr
+from Mailman import Pending
 
 # gui components package
 from Mailman import Gui
@@ -64,7 +65,6 @@ from Mailman import Gui
 from Mailman import MemberAdaptor
 from Mailman.OldStyleMemberships import OldStyleMemberships
 from Mailman import Message
-from Mailman import Pending
 from Mailman import Site
 from Mailman import i18n
 from Mailman.Logging.Syslog import syslog
@@ -84,7 +84,7 @@ except NameError:
 # Use mixins here just to avoid having any one chunk be too large.
 class MailList(HTMLFormatter, Deliverer, ListAdmin,
                Archiver, Digester, SecurityManager, Bouncer, GatewayManager,
-               Autoresponder, TopicMgr):
+               Autoresponder, TopicMgr, Pending.Pending):
 
     #
     # A MailList object's basic Python object model support
@@ -251,7 +251,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         if name:
             self._full_path = Site.get_listpath(name)
         else:
-            self._full_path = None
+            self._full_path = ''
         # Only one level of mixin inheritance allowed
         for baseclass in self.__class__.__bases__:
             if hasattr(baseclass, 'InitTempVars'):
@@ -366,7 +366,6 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         # 2-tuple of the date of the last autoresponse and the number of
         # autoresponses sent on that date.
         self.hold_and_cmd_autoresponses = {}
-
         # Only one level of mixin inheritance allowed
         for baseclass in self.__class__.__bases__:
             if hasattr(baseclass, 'InitVars'):
@@ -568,7 +567,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
                 if type(dict) <> DictType:
                     return None, 'Load() expected to return a dictionary'
             except (EOFError, ValueError, TypeError, MemoryError,
-                    cPickle.PicklingError), e:
+                    cPickle.PicklingError, cPickle.UnpicklingError), e:
                 return None, e
         finally:
             fp.close()
@@ -576,7 +575,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         self.__timestamp = mtime
         return dict, None
 
-    def Load(self, check_version=1):
+    def Load(self, check_version=True):
         if not Utils.list_exists(self.internal_name()):
             raise Errors.MMUnknownListError
         # We first try to load config.pck, which contains the up-to-date
@@ -609,13 +608,20 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
             raise Errors.MMCorruptListDatabaseError, e
         # Now, if we didn't end up using the primary database file, we want to
         # copy the fallback into the primary so that the logic in Save() will
-        # still work.  For giggles, we'll copy it to a safety backup.
-        if file == plast:
-            shutil.copy(file, pfile)
-            shutil.copy(file, pfile + '.safety')
-        elif file == dlast:
-            shutil.copy(file, dfile)
-            shutil.copy(file, pfile + '.safety')
+        # still work.  For giggles, we'll copy it to a safety backup.  Note we
+        # MUST do this with the underlying list lock acquired.
+        if file == plast or file == dlast:
+            syslog('error', 'fixing corrupt config file, using: %s', file)
+            unlock = True
+            try:
+                try:
+                    self.__lock.lock()
+                except LockFile.AlreadyLockedError:
+                    unlock = False
+                self.__fix_corrupt_pckfile(file, pfile, plast, dfile, dlast)
+            finally:
+                if unlock:
+                    self.__lock.unlock()
         # Copy the loaded dictionary into the attributes of the current
         # mailing list object, then run sanity check on the data.
         self.__dict__.update(dict)
@@ -623,6 +629,36 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
             self.CheckVersion(dict)
             self.CheckValues()
 
+    def __fix_corrupt_pckfile(self, file, pfile, plast, dfile, dlast):
+        if file == plast:
+            # Move aside any existing pickle file and delete any existing
+            # safety file.  This avoids EPERM errors inside the shutil.copy()
+            # calls if those files exist with different ownership.
+            try:
+                os.rename(pfile, pfile + '.corrupt')
+            except OSError, e:
+                if e.errno <> errno.ENOENT: raise
+            try:
+                os.remove(pfile + '.safety')
+            except OSError, e:
+                if e.errno <> errno.ENOENT: raise
+            shutil.copy(file, pfile)
+            shutil.copy(file, pfile + '.safety')
+        elif file == dlast:
+            # Move aside any existing marshal file and delete any existing
+            # safety file.  This avoids EPERM errors inside the shutil.copy()
+            # calls if those files exist with different ownership.
+            try:
+                os.rename(dfile, dfile + '.corrupt')
+            except OSError, e:
+                if e.errno <> errno.ENOENT: raise
+            try:
+                os.remove(dfile + '.safety')
+            except OSError, e:
+                if e.errno <> errno.ENOENT: raise
+            shutil.copy(file, dfile)
+            shutil.copy(file, dfile + '.safety')
+
 
     #
     # Sanity checks
@@ -702,7 +738,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         # admin approval, even if the list is so configured.  The flag is the
         # list name to prevent invitees from cross-subscribing.
         userdesc.invitation = self.internal_name()
-        cookie = Pending.new(Pending.SUBSCRIPTION, userdesc)
+        cookie = self.pend_new(Pending.SUBSCRIPTION, userdesc)
         confirmurl = '%s/%s' % (self.GetScriptURL('confirm', absolute=1),
                                 cookie)
         listname = self.real_name
@@ -815,7 +851,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         elif self.subscribe_policy == 1 or self.subscribe_policy == 3:
             # User confirmation required.  BAW: this should probably just
             # accept a userdesc instance.
-            cookie = Pending.new(Pending.SUBSCRIPTION, userdesc)
+            cookie = self.pend_new(Pending.SUBSCRIPTION, userdesc)
             # Send the user the confirmation mailback
             if remote is None:
                 by = remote = ''
@@ -1009,8 +1045,8 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         if newaddr == self.GetListEmail().lower():
             raise Errors.MMBadEmailError
         # Pend the subscription change
-        cookie = Pending.new(Pending.CHANGE_OF_ADDRESS,
-                             oldaddr, newaddr, globally)
+        cookie = self.pend_new(Pending.CHANGE_OF_ADDRESS,
+                               oldaddr, newaddr, globally)
         confirmurl = '%s/%s' % (self.GetScriptURL('confirm', absolute=1),
                                 cookie)
         realname = self.real_name
@@ -1078,7 +1114,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
     # Confirmation processing
     #
     def ProcessConfirmation(self, cookie, context=None):
-        rec = Pending.confirm(cookie)
+        rec = self.pend_confirm(cookie)
         if rec is None:
             raise Errors.MMBadConfirmation, 'No cookie record for %s' % cookie
         try:
@@ -1183,7 +1219,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
                 else:
                     # The password didn't match.  Re-pend the message and
                     # inform the list moderators about the problem.
-                    Pending.repend(cookie, rec)
+                    self.pend_repend(cookie, rec)
                     raise Errors.MMBadPasswordError
             else:
                 action = mm_cfg.DISCARD
@@ -1204,7 +1240,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
     def ConfirmUnsubscription(self, addr, lang=None, remote=None):
         if lang is None:
             lang = self.getMemberLanguage(addr)
-        cookie = Pending.new(Pending.UNSUBSCRIPTION, addr)
+        cookie = self.pend_new(Pending.UNSUBSCRIPTION, addr)
         confirmurl = '%s/%s' % (self.GetScriptURL('confirm', absolute=1),
                                 cookie)
         realname = self.real_name
